package org.silex.core;

import org.silex.publication.LayerParser;

#if php
import org.silex.htmlGenerator.InterpreterContext;
#end
/**
*  This class is used to execute SILEX commands.
*  It is supposed to be as close as possible to the AS2 version but targetting several platforms.
*  This is WIP.
*/
class Interpreter
{
	/**
	*  Stores all actions that are "on the stage". It is used for example when handling events.
	*/
	public static var actions = new Array<{component : String,modifier : String,functionName : String,parameters : Array<String>}>();
	/**
	*  Basic implementation of the singleton pattern. Maybe a static implementation would fit better now that
	*  we don't need to support scope modification anymore in actions.
	*/
	public static var interpreter = new Interpreter();
	
	//Note that initialSource is Dynamic since it could __potentially__ be any object.
	//This way we allow actions on objects that do not have a visual representation.
	public static function exec(command : String, initialSource : Dynamic) : Bool
	{
		if(command == null || command == "")
		{
			return false;
		}
		
		// /////////////////////////
		// = operator
		// replace x.y=z by x.set:y,z
		var equalOperatorIndex : Int = command.indexOf("=");
		var columnOperatorIndex : Int= command.indexOf(":");
		
		// if there is "=" and no ":"
		// or a "=" before any ":"
		if ((equalOperatorIndex>=0 && columnOperatorIndex<0) || (equalOperatorIndex>0 && equalOperatorIndex<columnOperatorIndex)){
			var res_str : String="";
			// extract x.y
			var targetVariablePath_str : String;
			targetVariablePath_str = command.substr(0,equalOperatorIndex);
			// insert "=" between x and y
			var targetVariableIdx : Int;
			targetVariableIdx = targetVariablePath_str.lastIndexOf(".")+1;
			if (targetVariableIdx>0)
				res_str=targetVariablePath_str.substr(0,targetVariableIdx);
			res_str+="set:"+targetVariablePath_str.substr(targetVariableIdx)+",";

				
			// extract z
			var value_str:String=command.substr(equalOperatorIndex+1);
			// transform it to x.set:y
			res_str+=value_str;
			
			command=res_str;
		}
		//trace(command);
		
		// /////////////////////////
		// separate the command and the arguments
		var command_array:Array<String>;
		command_array = extract(command);

		// separate method from source path and get the source object
		var methodName_str:String;
		var source_mc:Dynamic;
		#if js
		source_mc = js.Lib.document.body;
		#end
		#if php
		source_mc = InterpreterContext.components;
		#end
		var lastDotIndex : Int=command_array[0].lastIndexOf(".");
		if (lastDotIndex>0){
			// separate method from source path
			var relativePath_str:String=command_array[0].substr(0,lastDotIndex);
			methodName_str=command_array[0].substr(lastDotIndex+1);
			
			// get the source object
			//**TODO** : Implement
			source_mc=updateSource(initialSource,relativePath_str);
		}
		else{
			methodName_str=command_array[0];
			source_mc=initialSource;
		}
		
		// convert accessors into values in the command and in each parameter
		
		for (idx in 0...command_array.length)
		{
			command_array[idx] = Utils.revealAccessors(command_array[idx],source_mc);
		}
		
		
		// build params_array
		var params_array : Array<Dynamic>;
		params_array = command_array;
		// remove command from params_array
		params_array.shift();

		// here we have:
		// source_mc			relative.path.to.source object
		// methodName_str		method
		// params_array			[param1,param2,...]
		//TODO not sure this is right. Maybe better to do only one return and use that?

		// call the method on the source object
		//trace(source_mc);
		if (Reflect.hasField(source_mc, methodName_str)){
			return Reflect.callMethod(source_mc, Reflect.field(source_mc, methodName_str), params_array);
//			return true;
		}
		// call the method on the interpreter
		if (Lambda.has(Type.getInstanceFields(Interpreter), methodName_str)){
			//Add execution context as first parameter
			params_array.insert(0, source_mc);
			#if php
			//In PHP we need to passs the exact number of required arguments.
			//So we are going to retrieve the number of needed arguments, and add nulls as needed to our array.
			var classRefl : Dynamic = untyped __php__("new ReflectionClass('org_silex_core_Interpreter')");
			var neededParams : Int = classRefl.getMethod(methodName_str).getNumberOfRequiredParameters();

			//Actually add parameters
			for(i in 0...neededParams- params_array.length)
			{
				params_array.push(null);
			}
			#end
			return Reflect.callMethod(Interpreter.interpreter, Reflect.field(Interpreter.interpreter, methodName_str), params_array);
		}
			
		return null;
	}
	
	public function new()
	{}
	
	private static function updateSource (source_mc:Dynamic,path_str:String):Dynamic {
		// find the movie clip corresponding to the path relatively from the source
		if (path_str != null){
			var tmp_mc:Dynamic;

			//if (!(tmp_mc=silex_ptr.application.getPlayerByName(path_str)))
				tmp_mc = Utils.getTarget(source_mc,path_str);

			// if it was found, take it as the source
			if (tmp_mc != null) source_mc=tmp_mc;
		}
		return source_mc;
	}
	
	public static function extract(command : String) : Array<String>
	{
		var ret_array : Array<String>;
		ret_array =new Array<String>();

        if (command == null || command == "")
            return null;
            

        var indice_num=command.indexOf(":");
        if (indice_num>=0)
        {// Command with a ':'
            // command extraction
            ret_array[0]=command.substr(0,indice_num);

            // params extraction
            var tmp_array : Array<String>;
			tmp_array = command.substr(indice_num+1).split(",");
            // Concat in the result array
            for(i in 0...tmp_array.length)
            {
                ret_array[i+1]=tmp_array[i];
            }
        }
        else
        {// Command without a ':' => no params
            ret_array[0]=command;
        }
		//trace(ret_array);
        return ret_array;
	}
	
	
	/**
	*  Opens an URL
	*/
	public function openUrl(context : Dynamic, url : String, ?target : String)
	{
		//###Parameters' default values
		//Set default value of target
		if(target == null)
			target = "_blank";
		
		//###Flash implementation
		#if flash
		flash.Lib.getUrl(url, target);
		#end
		
		//###JS implementation
		#if js
		//Note that this could maybe be improved using some tricks
		//in order to have "_blank" feel more like if you had clicked
		//on a link and less like a popup
		js.Lib.window.open(url, target);
		#end
		
		#if php
		if(Reflect.hasField(context, "__silex__actionshref__"))
			context.__silex__actionshref__ = url;
		#end
	}
	
	/**
	*  Opens an URL with the HTTP protocol.
	*/
	public function http(context : Dynamic, url : String, ?target : String)
	{
		//###Parameters' default values
		//Set default value of target
		if(target == null)
			target = "_blank";
		
		//Simply call openUrl but appending http: before
		openUrl(context, "http:" + url, target);
	}
	
	/**
	*  Opens the default mail application to send a mail.
	*/
	public function mailto(context : Dynamic, address : String)
	{
		//Simply call openUrl but appending mailto: before
		openUrl(context, "mailto:" + address, "_self");
	}
	
	public function download(context : Dynamic, initialFileName : String, ?finalFileName : String)
	{
		//###Parameters' default values
		//Set default value of finalFileName
		if(finalFileName != null)
			finalFileName = initialFileName.split("/").pop();
		
		//TODO: Implement
	}
	
	public static function javascript(context : Dynamic, code : String)
	{
		//###JS implementation
		#if js
			js.Lib.eval(code);
		#end
		
		//TODO: Implement in Flash
	}
	
	public function set(context : Dynamic, propertyName : String, valueStr : String)
	{
		#if js
		//We've got to do some transformation (_x => style.left for examples)
		switch(propertyName)
		{
			case "_x":
				untyped context.style.left = valueStr + "px";
			case "_y":
				untyped context.style.top = valueStr + "px";
			case "rotation":
				//**TODO** : Implement
				untyped context.style.webkitTransform = "rotate(" + valueStr + "deg)";
			case "_width":
				untyped context.style.width = valueStr + "px";
			case "_height":
				untyped context.style.height = valueStr + "px";
			case "url":
				//Try to determine if we are working on a DOM element representing a component
				if(context.nodeName == "DIV")
				{
					//Is it an image component?
					if(context.firstChild.nodeName == "IMG")
						context.firstChild.src = valueStr;
					else
						context.url = valueStr;
				} else
					context.url = valueStr;
			default:
				//**TODO**: There are quite a lot of type conversion that should be done here.
				Reflect.setField(context, propertyName, valueStr);
		}
		#end
	}
	
	public function del(context : Dynamic, propertyName : String)
	{
		#if js
		//We've got to do some transformation (_x => style.left for examples)
		switch(propertyName)
		{
			case "_x":
				//Can we remove style.left?
			case "_y":
				//Can we remove style.top?
			case "rotation":
				//Just remove CSS style?
			case "_width":
				//Can we remove style.width?
			case "_height":
				//Can we remove style.height?
			default:
				//**TODO**: There are quite a lot of type conversion that should be done here.
				Reflect.deleteField(context, propertyName);
		}
		#end
	}
	
	/**
	*  Shows a component.
	*/
	public static function show(context : Dynamic, targetObjectPath : String)
	{
		//TODO: Implement
		//DEPENDSON: Path solving
	}

	/**
	*  Hides a component.
	*/
	public function hide(context : Dynamic, targetObjectPath : String)
	{
		#if js
		var source_mc = updateSource(context, targetObjectPath);
		source_mc.style.display = "none";
		//TODO: Implement
		//DEPENDSON: Path solving
		#end
	}
	
	public function alert(context : Dynamic, text : String)
	{
		//###JS implementation
		#if js
		js.Lib.alert(text);
		#end
		
		//###Flash implementation
		#if flash
		//TODO: Implement
		//DEPENDSON: Utils
		#end
		
		#if php
		#end
	}
	
	/**
	*  This has been added for debugging purpose. It simply traces "this".
	*/
	public function traceMe(context : Dynamic)
	{
		#if php
		trace("traceMe:" + Std.string(context));
		#end
		#if js
		js.Lib.alert("traceMe:" + Std.string(context));
		#end
	}
	
	public static function alertSimple(context : Dynamic, text : String)
	{
		//TODO: Implement
	}
	
	/**
	*  Opens the specified website on the same server.
	*/
	public static function openWebsite(context : Dynamic, idSite: String)
	{
		//###JS implementation
		#if js
			js.Lib.window.location.replace(js.Lib.window.location.href.split("?")[0] + "?/" + idSite);
		#end
		#if php
		//**TODO**: Implement
		#end
	}
	
	/**
	*  Opens a part of a publication
	*/
	public function open(context : Dynamic, sectionName : String)
	{
		if(sectionName.charAt(0) == "/")
		{
			sectionName = sectionName.substr(1, sectionName.length-1);
		}
		var layers : Array<String>;
		layers = sectionName.split("/");
		//If first part is not the start section prepend it
		//**TODO** : WEBSITE_CONFIG_START has been hardcoded due to the impossibility of getting website config from JS
		//at the moment
		var startLayer : String = "start";
		#if php
		startLayer = InterpreterContext.websiteConfig.get("CONFIG_START_SECTION");
		#end
		if(layers[0] != startLayer)
		{
			layers.insert(0, startLayer);
		}
		//Get the publication name
		var publicationName : String = "";
		#if js
		if(js.Lib.window.location.search.indexOf("?/") == 0)
		{
			publicationName = js.Lib.window.location.search.split("/")[1];
		}
		//If it only starts with ?
		else if(js.Lib.window.location.search.indexOf("?") == 0)
		{
			publicationName = js.Lib.window.location.search.split("/")[0].substr(1);
		}
		#end
		#if php
		publicationName = org.silex.htmlGenerator.Utils.deeplink.publication;
		#end
		
		#if js
		//Move to new location
		//js.Lib.window.location.replace("?/" + publicationName + "/" + layers.join("/") + "&format=html");
		untyped js.Lib.window.location = "?/" + publicationName + "/" + layers.join("/") + "&format=html";
		#end
		#if php
		openUrl(context, "?/" + publicationName + "/" + layers.join("/") + "&format=html", "_self");
		#end
	}
	
	/**
	*  This function can be used to initialize action handling.
	*  For example, it is used in JS to retrieve data from the global var storing actions and initializing
	*  events handlers.
	*/
	public static function initActions()
	{
		#if js
		Interpreter.actions = haxe.Unserializer.run(untyped js.Lib.window.silexActions);
		for(action in actions)
		{
			switch(action.modifier)
			{
				case "onRelease", "onPress":
					untyped js.Lib.document.getElementById(action.component).onclick = function (e : js.Dom.Event) {
						Interpreter.handleEvent("onRelease", action.component);
						//Cancel browser's default action (prevents navigation)
						return false;
					};
				default:
			}
		}
		#end

		#if php
		for(action in actions)
		{
			switch(action.modifier)
			{
				case "onRelease", "onPress":
					//In PHP, we do not wait for the event to happen as we modify what will happen when the event
					//is going to be triggered.
					Interpreter.handleEvent(action.modifier, action.component);
				default:
				//The static HTML version does know how to handle actions others than onRelease
			}
		}
		#end

	}
	
	/**
	*  This function is used to handle events when they happen and trigger the appropriate actions.
	*  Note that event should be the name of a silex event("modifier") such as onRelease.
	*/
	public static function handleEvent(event : String, component : String)
	{
		for(action in actions)
		{
			if(event == action.modifier && component == action.component)
			{
				var silexCode = action.functionName;
				//We have to htmlUnescape every parameter
				for(i in 0...action.parameters.length)
				{
					action.parameters[i] = StringTools.htmlUnescape(action.parameters[i]);
				}
				if(action.parameters.length > 0)
				{
					//Reconstructs the "silex code"
					silexCode += ":" + action.parameters.join(",");
				}
				#if js
				Interpreter.exec(silexCode, js.Lib.document.getElementById(action.component));
				#end
				#if php
				Interpreter.exec(silexCode, InterpreterContext.components.get(action.component));
				#end
			}
		}
	}
}