/*This file is part of Silex - see http://projects.silexlabs.org/?/silex

Silex is © 2010-2011 Silex Labs and is released under the GPL License:

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License (GPL) as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

To read the license please visit http://www.gnu.org/copyleft/gpl.html
*/
package org.silex.htmlGenerator;

import haxe.Resource;
import haxe.Template;
import org.silex.publication.LayerParser;
import org.silex.serverApi.ComponentManager;
import php.io.File;
import php.Lib;
import php.Web;
import org.silex.serverApi.ServerConfig;
import org.silex.serverApi.SiteEditor;
import org.silex.serverApi.helpers.Env;
import org.silex.serverApi.RootDir;
import org.silex.serverApi.ServerContent;
import org.silex.serverApi.PluginManager;
import org.silex.serverApi.HookManager;
import org.silex.serverApi.ConfigEditor;
import org.silex.serverApi.FileSystemTools;
import org.silex.serverApi.FileSystemItem;
import org.silex.serverApi.Logger;
import org.silex.htmlGenerator.JsCommunication;
import org.silex.core.Interpreter;

import org.silex.htmlGenerator.exceptions.LayerException;

using org.silex.filters.FilterManager;

/**
*  This is the Main class of the HTML5Generator and contains the entry point for server-side generator.
*/
class Main
{
	static function noFlashJS(idSite : String, deeplink : String)
	{
		var websiteUrl = "'?" + idSite + "/&format=html'";
		var scriptTag = "<script type='text/javascript'>
		$no_flash_page=" + websiteUrl + ";
		";
	}
	
	static function noFlashNoJS(idSite : String, deeplink : String)
	{
		Lib.print("<p><a href='" + "?" + idSite + "/&format=html'>You do not have Flash. You can try to access the HTML version of this site by clicking here.</a></p>");
	}
	
	/**
	*  Entry point. Sets-up filters and hooks. It is indeed called on initHooks (via index.php).
	*/
	static function main()
	{
		"template-index-head".addFilter(templateHeadFilter, 1000);
		"template-index-body".addFilter(templateBodyFilter, 1000);
		"template-index-footer".addFilter(templateFooterFilter, 1000);
		"template-context".addFilter(templateContextFilter, 0);
		"index-remoting-context".addFilter(remotingContextFilter, 0);
	}
	
	/**
	*  Filters the head template's name. Used to replace the Flash's one by ours if using HTML Renderer.
	*/
	static function templateHeadFilter(templateName : String, context : Dynamic)
	{
		if(context.renderer == "html")
			return "plugins/htmlGenerator/generator/res/head.html";
		else
			return templateName;
	}
	
	/**
	*  Filters the body template's name. Used to replace the Flash's one by ours if using HTML Renderer.
	*/
	static function templateBodyFilter(templateName : String, context : Dynamic)
	{
		if(context.renderer == "html")
			return "plugins/htmlGenerator/generator/res/body.html";
		else
			return templateName;
	}
	
	/**
	*  Filters the footer template's name. Used to replace the Flash's one by ours if using HTML Renderer.
	*/
	static function templateFooterFilter(templateName : String, context : Dynamic)
	{
		if(context.renderer == "html")
			return "plugins/htmlGenerator/generator/res/footer.html";
		else
			return templateName;
	}
	
	/**
	*  Filters the context given to templates when executing them.
	*  Used to pass the rendering result (such as htmlCode) to our template.
	*  It also runs the rendering **only if using HTML Renderer**.
	*/
	static function templateContextFilter(templateContext : Dynamic, context : Dynamic)
	{
		templateContext.isIPad = ~/iPad/i.match(untyped __var__("_SERVER", "HTTP_USER_AGENT")) || ~/iPhone/i.match(untyped __var__("_SERVER", "HTTP_USER_AGENT"));
		//If we are using the HTML renderer.
		if(templateContext.renderer == "html")
		{
			var renderResult = {redirectTo : "", height: 0, width: 0, bgColor : "", cssCode : "", htmlCode: "", jsCode: ""};
			try
			{
				//Call rendering engine.
				renderResult = new RenderingEngine().render(Web.getParamsString());
				//Add SILEXStage to it.
				renderResult.htmlCode = '<div class="SILEXStage" style="position:relative;margin-left:auto;margin-right:auto;width:' + renderResult.width + 'px;height:' + renderResult.height + 'px;background-color:#' + renderResult.bgColor + '">'
								+ renderResult.htmlCode + "</div>";
				//We want to prevent redirection to self.
				if(renderResult.redirectTo != null && renderResult.redirectTo != "?" + Web.getParamsString())
				{
					Web.setHeader("Location", renderResult.redirectTo);
				}
			} catch(e : LayerException)
			{
				switch(e)
				{
					case LayerException.notFound(layer):
						renderResult.htmlCode = "<span class='error'>" + layer + " layer could not be found.</span>";
					case LayerException.parseError(layer):
						renderResult.htmlCode = "<span class='error'>" + layer + " layer is malformed.</span>";
				}
			}

			//Populate templateContext with our data.
			templateContext.bgColor = renderResult.bgColor;
			templateContext.cssCode = renderResult.cssCode;
			templateContext.htmlCode = renderResult.htmlCode;
			templateContext.jsCode = renderResult.jsCode;
			return templateContext;
		} else
		{
			return templateContext;
		}
	}
	
	/**
	*  Filters called to add our remoting object to the remoting context.
	*  It allows us to make RenderingEngine available via Remoting.
	*/
	static function remotingContextFilter(remotingContext : haxe.remoting.Context, context : Dynamic)
	{
		remotingContext.addObject("HTMLRenderingEngine", new RenderingEngine());
		return remotingContext;
	}
}