package org.silex.htmlGenerator;

import haxe.Resource;
import haxe.Template;
import org.silex.publication.LayerParser;
import org.silex.serverApi.ComponentManager;
import php.io.File;
import php.Lib;
import php.Web;
import org.silex.serverApi.ServerConfig;
import org.silex.serverApi.SiteEditor;
import org.silex.serverApi.helpers.Env;
import org.silex.serverApi.RootDir;
import org.silex.serverApi.ServerContent;
import org.silex.serverApi.PluginManager;
import org.silex.serverApi.HookManager;
import org.silex.serverApi.ConfigEditor;
import org.silex.serverApi.FileSystemTools;
import org.silex.serverApi.FileSystemItem;
import org.silex.serverApi.Logger;
import org.silex.htmlGenerator.JsCommunication;
import org.silex.core.Interpreter;

import org.silex.htmlGenerator.exceptions.LayerException;

/**
*  This RenderingEngine outputs HTML code for the HTML Renderer.
*/
class RenderingEngine
{
	public function new(){}
	private static var loadedHtml5Modules = new Array<String>();
	
	/**
	*  This function returns a website's configuration as Hash<String>
	*/
	public function getPublicationConfig(publicationName : String) : Hash<String>
	{
		var siteEditor = new SiteEditor();
		return siteEditor.getWebsiteConfig(publicationName);
	}
	/**
	*  This function does the actual rendering of the page.
	*  It returns an object containing HTML code, JS code, CSS code and the background-color.
	*/	
	public function render(url : String)
	{
		//Those are pathes needed by the SILEX classes
		Env.setIncludePath(Env.getIncludePath() + Env.pathSeparator + "./");
		Env.setIncludePath(Env.getIncludePath() + Env.pathSeparator + "./cgi/library");
		Env.setIncludePath(Env.getIncludePath() + Env.pathSeparator + "./cgi/includes");
		
		//##Generator Initialization
		
		//Parse the Deeplink.
		var deeplink = Deeplink.parseURL(url);
		Utils.deeplink = deeplink;
		
		//This variable is used to store initialized sublayers and the components inside them.
		//This is useful in order to be able to iterate over components on stage in the rendering order(z-order).
		var initializedSubLayers : Array<SubLayer> = new Array<SubLayer>();
		
		//Initialize some needed classes
		var configEditor = new ConfigEditor();
		var serverConfig = new ServerConfig();
		//Get useful information about the publication
		var contentsPath = serverConfig.getContentFolderForPublication(deeplink.publication);
		var silexPluginsConf = configEditor.readConfigFile(serverConfig.silexServerIni.get("SILEX_PLUGINS_CONF"), "phparray");
		var hookManager = HookManager.getInstance();
		var siteEditor = new SiteEditor();
		Utils.siteEditor = siteEditor;
		InterpreterContext.websiteConfig = siteEditor.getWebsiteConfig(deeplink.publication);
		var width : Int = Std.parseInt(siteEditor.getWebsiteConfig(deeplink.publication).get("layoutStageWidth"));
		var height : Int = Std.parseInt(siteEditor.getWebsiteConfig(deeplink.publication).get("layoutStageHeight"));
		var bgColor : String = siteEditor.getWebsiteConfig(deeplink.publication).get("bgColor");
		//Holds redirection that has to be applied.
		var redirectTo : String;
		redirectTo = null;
		//Redirect to start section if no deeplink is provided.
		if (deeplink.layers.length == 0)
			redirectTo = "?/" + deeplink.publication + "/" + siteEditor.getWebsiteConfig(deeplink.publication).get("CONFIG_START_SECTION")+"&format=html";
			//Web.setHeader("Location", "?/" + deeplink.publication + "/" + siteEditor.getWebsiteConfig(deeplink.publication).get("CONFIG_START_SECTION")+"&format=html");
		
		//##Output generation
		
		//We initialize the htmlCode to be output as an empty string.
		var htmlCode = "";
		var layersHTMLEquivalent = new Hash<String>();

		//Create the silex stage.
		//htmlCode += '<div class="SILEXStage" style="position:relative;margin-left:auto;margin-right:auto;width:' + width + 'px;height:' + height + 'px;background-color:#' + bgColor + '">';

		//###Layers generation
		
		//Those two vars are useful for icon and default icon management
		//layerCount is used to keep track of the *deepness* of the layer we are handling at the moment.
		//layerPath is used to keep track of the *path* of the layer we are handling at the moment.
		var layerCount = 0;
		var layerPath = new Array<String>();

		//Generate all layers
		for (layer in deeplink.layers)
		{
			//Define layer's path.
			layerPath.push(layer);
			//Define layer's deepness
			layerCount++;
			
			var xmlContent : String = null;
			try
			{
				xmlContent = File.getContent(contentsPath + deeplink.publication + "/" + layer + ".xml");
			} catch(e : Dynamic)
			{
				continue;
				//We changed behavior: now we simply ignore layers that we can not open.
				//throw LayerException.notFound(layer);
			}
			var parsedLayer = null;
			try
			{
				parsedLayer = LayerParser.xml2Layer(Xml.parse(xmlContent));
			} catch(e : Dynamic)
			{
				throw LayerException.parseError(layer);
			}
			parsedLayer.name = layer;
			//Order SubLayers by zIndex
			var orderedSubLayers : Array<org.silex.publication.SubLayerModel>;
			orderedSubLayers = Lambda.array(parsedLayer.subLayers);
			/*
			orderedSubLayers.sort(function(arg1: org.silex.publication.SubLayerModel, arg2: org.silex.publication.SubLayerModel)
			{
				if(! (Reflect.hasField(arg1, "zIndex") && Reflect.hasField(arg2, "zIndex")))
				{
					trace(arg1.id + " has no zIndex in " + parsedLayer.name);
					return 0;
				}
				if(arg1.zIndex == arg2.zIndex)
				{
					return 0;
				} else if(arg1.zIndex > arg2.zIndex)
				{
					return 1;
				} else
				{
					return -1;
				}
			});
			*/
			//####SubLayers generation
			for (subLayer in orderedSubLayers)
			{
				//Create the SubLayer instance for this subLayer and add it to the list of initializedSubLayers
				var subLayerInstance = new SubLayer();
				subLayerInstance.layerName = parsedLayer.name;
				subLayerInstance.subLayerName = subLayer.id;
				initializedSubLayers.push(subLayerInstance);
				
				var comps = Lambda.array(subLayer.components);

				//#####Components initialization
				//We've got to reverse components array to have them in the same order as in SILEX Flash
				comps.reverse();
				for (componentModel in comps)
				{
					//Module loading.
					if(StringTools.trim(componentModel.html5Url) != "")
					{
						var pathToLoad : String;
						pathToLoad = org.silex.serverApi.RootDir.rootPath + componentModel.html5Url.split("#")[0];
						pathToLoad = StringTools.trim(pathToLoad); //Remove bad spaces
						if(pathToLoad.charAt(pathToLoad.length-1) == "/")
						{
							pathToLoad = pathToLoad.substr(0, pathToLoad.length-1);
						}
						if(StringTools.trim(pathToLoad) != "")
						{
							//Do not try to reload modules that are already loaded
							if(!Lambda.has(loadedHtml5Modules, pathToLoad))
							{
								//Actually loads the components in the package.
								php.Lib.loadLib(pathToLoad);
								loadedHtml5Modules.push(pathToLoad);
							}
						}
					}

					//Get the name of the class from the html5Url.
					var className = componentModel.html5Url.split("#")[1];

					//Create the instance and set some information about its context.
					var componentInstance : Dynamic = null;
					try {
						componentInstance = Type.createInstance(Type.resolveClass(className), []);
					} catch(e : Dynamic)
					{
						//Lib.println("Could not create instance of " + className);
						componentInstance = new org.silex.ui.UiBase();
					}
					componentInstance.__silex__layerPath__ = layerPath.copy();
					componentInstance.__silex__publicationName__ = deeplink.publication;
					//WARNING: If you need to access the component's properties, use the c.properties table since not all properties are already initialized.
					for (propertyName in componentModel.properties.keys())
					{
						if (Reflect.hasField(componentInstance, propertyName))
							Reflect.setField(componentInstance, propertyName, componentModel.properties.get(propertyName));
						//If icon is default icon, do the redirection IF NEEDED (no "under layer" already opened)
						if (propertyName == "iconIsDefault" && componentModel.properties.get(propertyName) == true && layerCount == deeplink.layers.length)
						{
							//Icon is default, redirect to its deeplink
							if (componentInstance.iconDeeplinkName != "" && componentInstance.iconDeeplinkName != null)
							{
								redirectTo = "?/" + deeplink.publication + "/" + componentModel.properties.get("iconDeeplinkName") + "&format=html&selectedIcon=" + componentInstance.__silex__layerPath__.join(".") + "." + componentInstance.playerName;
								//Web.setHeader("Location", redirectTo);
							} else
							{
								redirectTo = "?/" + deeplink.publication + "/" + layerPath.join("/") + "/" + componentModel.properties.get("iconPageName") + "&format=html&selectedIcon=" + componentInstance.__silex__layerPath__.join(".") + "." + componentInstance.playerName;
								//Web.setHeader("Location", redirectTo);
							}
						}
					}
					//The component is now totally initialized so add it to the list of components of this subLayer instance
					subLayerInstance.initializedComponents.push({ componentModel : componentModel, componentInstance : componentInstance });

					//Register this component with its playerName so that it is available for scripting.
					InterpreterContext.components.set(componentInstance.playerName, componentInstance);
					
					//If this component is an icon, add an open action to it
					if(componentInstance.iconIsIcon)
					{
						var action = new org.silex.publication.ActionModel();
						action.functionName = "open";
						action.modifier = "onRelease";
						var params = new List<String>();
						//Set the correct url to open depending on wether deeplinkName is set or not.
						if(componentInstance.iconDeeplinkName =="" || componentInstance.iconDeeplinkName == null)
						{
							params.add(componentInstance.__silex__layerPath__.join("/") + "/" + componentInstance.iconPageName + "&amp;format=html&amp;selectedIcon=" + StringTools.urlEncode(componentInstance.__silex__layerPath__.join(".") + "." + componentInstance.playerName));
						} else
						{
							params.add(componentInstance.iconDeeplinkName + "&amp;format=html&amp;selectedIcon=" + StringTools.urlEncode(componentInstance.__silex__layerPath__.join(".") + "." + componentInstance.playerName));
						}
						action.parameters = params;
						componentModel.actions.add(action);
					}
				}
			}
			//Component initialization and properties initialization is done.
			

		}
		//####Actions handling
		for(subLayer in initializedSubLayers)
		{
			for(component in subLayer.initializedComponents)
			{
				//This block of code creates the representation of actions from the component and
				//pushes them to an array that will be serialized and passed to the JSRuntime.
				for(action in component.componentModel.actions)
				{
					var action = 	{component : component.componentInstance.playerName,
									modifier : action.modifier,
									functionName : action.functionName,
									parameters : Lambda.array(action.parameters)};
					JsCommunication.actions.push(action);
					Interpreter.actions.push(action);
				}
			}
		}
		//Now apply actions using the Interpreter.
		Interpreter.initActions();
		
		//####HTML Rendering
		for(subLayer in initializedSubLayers)
		{
			try
			{
				htmlCode += subLayer.returnHTML();
			}catch (e : Dynamic)
			{
				htmlCode += "EXCEPTION - SubLayer: " + subLayer.layerName+ ") " + Std.string(e);
			}
		}
		
		//htmlCode += '</div>';
		//Execute template and print everything
		var jsCode = "window.silexActions='" + haxe.Serializer.run(JsCommunication.actions) +"';";
		//Lib.print(new Template(Resource.getString("body.html")).execute( { bgColor: bgColor, htmlCode : htmlCode, cssCode : Utils.getCssText(), jsCode : jsCode } ));
		return { redirectTo : redirectTo, width: width, height: height, bgColor: bgColor, htmlCode : htmlCode, cssCode : Utils.getCssText(), jsCode : jsCode };
	}
}